#!/bin/sh
#----------------------------------------------------------------------------
# /var/install/config.d/mariadb55.sh - MariaDB/MySQL config script
#
# Creation:     2005-10-28 jv
# Last Update:  $Id: mariadb55.sh 106894 2023-10-19 14:26:17Z hbfl $
#
# Copyright (c) 2004 Jens Vehlhaber <jens(at)eisfair(dot)org>
# Copyright (c) 2011-2023 Holger Bruenjes, holgerbruenjes(at)gmx(dot)net
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#----------------------------------------------------------------------------

#include eisdate-time
. /var/install/include/eislib

package_name=mariadb55

VERSION=3.4.1

# ---------------------------------------------------------------------------
# Set the defaults
# ---------------------------------------------------------------------------
START_MARIADB55="yes"
MARIADB55_MEMORY_OPT="medium"
MARIADB55_BIND_IP_ADDRESS=""
MARIADB55_BACKUP_SCHEDULE_TIME='*-*-* 01:15:00'
MARIADB55_ACTIVATE_BINLOG="no"
MARIADB55_MAX_BINLOG_SIZE="1024000"
MARIADB55_MAX_CONNECTIONS='200'

_default_datadir="/srv/mysql/55"
MKTEMP=/usr/bin/mktemp
PW_TEST=/usr/bin/pwtest
CAT=/usr/bin/cat
RM=/usr/bin/rm
ASK=/var/install/bin/ask

timer_dir='/etc/systemd/system/mariadb55-backup.timer.d'
timer_conf='75-backup-time.conf'

# ---------------------------------------------------------------------------
# mariadb start
# ---------------------------------------------------------------------------
start_mariadb()
{
    if /usr/sbin/service is-enabled ${package_name}.service
    then
        /usr/sbin/service daemon-reload
    else
        /usr/sbin/service enable ${package_name}.service
    fi

    if "${backup_schedule:-false}"
    then
        if /usr/sbin/service is-enabled ${package_name}-backup.timer
        then
            /usr/sbin/service restart ${package_name}-backup.timer
        else
            /usr/sbin/service enable ${package_name}-backup.timer
            /usr/sbin/service start ${package_name}-backup.timer
        fi
    else
        /usr/sbin/service --quiet stop ${package_name}-backup.timer
        /usr/sbin/service disable ${package_name}-backup.timer
        /usr/bin/systemctl clean --what=state ${package_name}-backup.timer
    fi
}

# ---------------------------------------------------------------------------
# mariadb stop
# ---------------------------------------------------------------------------
stop_mariadb()
{
    /usr/sbin/service stop ${package_name}-backup.timer
    /usr/sbin/service disable ${package_name}.service
    /usr/sbin/service disable ${package_name}-backup.timer
    /usr/bin/systemctl clean --what=state ${package_name}-backup.timer
}

# ---------------------------------------------------------------------------
# remove schedule
# ---------------------------------------------------------------------------
remove_schedule()
{
    rm -f ${timer_dir}/${timer_conf}
    rmdir --ignore-fail-on-non-empty ${timer_dir} 2>/dev/null
}

# ---------------------------------------------------------------------------
# create schedule
# ---------------------------------------------------------------------------
create_schedule()
{
    default_schedule_time='*-*-* 01:15:00'

    if [ "${MARIADB55_BACKUP_SCHEDULE_TIME}" != "${default_schedule_time}" ]
    then
        # create timer job
        mkdir -p ${timer_dir}

        cat >${timer_dir}/${timer_conf} <<EOF
# ------------------------------------------------------------------------------
# ${timer_dir}/${timer_conf} generated by ${package_name} Version ${VERSION}
# 
#  Do not edit this file, edit /etc/config.d/${package_name}
#  Creation Date: ${EISDATE} Time: ${EISTIME}
# -------------------------------------------------------------------------------
[Timer]
OnCalendar=
OnCalendar=${MARIADB55_BACKUP_SCHEDULE_TIME}
EOF
    else
        remove_schedule
    fi
}

# ---------------------------------------------------------------------------
# add cron job for Backup
# ---------------------------------------------------------------------------
add_cron_job()
{
    mkdir -p /var/cron/etc/root

    if [ -f /var/cron/etc/root/mysql ]
    then
        rm -f /var/cron/etc/root/mysql
    fi

    if [ -f /var/cron/etc/root/mariadb ]
    then
        rm -f /var/cron/etc/root/mariadb
    fi

    # write mariadbdb cron file
    cat > /var/cron/etc/root/mariadb55  <<EOF
#----------------------------------------------------------------------------
# MariaDB/MySQL autobackup
# Do not edit this file, use /etc/config.d/mariadb55
# Creation: ${EISDATE} ${EISTIME}
#----------------------------------------------------------------------------
${MARIADB55_BACKUP_CRON_SCHEDULE} /usr/bin/mysql-common-backup.sh --project mariadb55 --all
EOF
    # update crontab file
    /var/install/config.d/cron --quiet
}

# ---------------------------------------------------------------------------
# delete cron job
# ---------------------------------------------------------------------------
delete_cron_job()
{
    # check for crontab file
    if [ -f /var/cron/etc/root/mariadb55 ]
    then
        # delete existing file
        rm -f /var/cron/etc/root/mariadb55
        # update crontab file
        /var/install/config.d/cron --quiet
    fi
}

# ---------------------------------------------------------------------------
# create my.cnf File
# ---------------------------------------------------------------------------
make_mysql_conf()
{
    mkdir -p /etc/mysql
    mkdir -p /etc/my55.cnf.d

    {
    echo "# =================================================================="
    echo "# MariaDB/MySQL configuration file"
    echo "# Do not edit this file! Use /etc/config.d/mariadb55"
    echo "# Creation: ${EISDATE} ${EISTIME} "
    echo "# =================================================================="
    echo "[client]"
    echo "port = ${MARIADB55_CONNECT_PORT}"
    echo "socket = /run/mysql/55/mysql.sock"
    echo
    echo "[mysqld_safe]"
    echo "socket = /run/mysql/55/mysql.sock"
    echo "nice = 0"
    echo
    echo "[mysqld]"
    echo "user = mysql"
    echo "pid-file = /run/mysql/55/mysql.pid"
    echo "socket = /run/mysql/55/mysql.sock"
    echo "basedir = /usr/lib/mysql/55"
    echo "datadir = ${_mysql_datadir}"
    echo "tmpdir = /tmp"
    echo "plugin-dir = /usr/lib/mysql/55/plugin"
    echo "lc-messages-dir = /usr/lib/mysql/55/share/mysql"
    echo "lc-messages = de_DE"
    echo 'character-sets-dir = /usr/lib/mysql/55/share/mysql'
    if [ "${MARIADB55_NETWORK}" != "yes" ]
    then
        echo "skip-networking"
    else
        if [ -n "${MARIADB55_BIND_IP_ADDRESS}" ]
        then
            echo "bind-address = ${MARIADB55_BIND_IP_ADDRESS}"
        fi
    fi
    echo "port = ${MARIADB55_CONNECT_PORT}"
    # System-Sperren nicht benutzen:
    echo "skip-external-locking"
    # < 64 MB RAM
    if [ "${MARIADB55_MEMORY_OPT}" = "small" ]
    then
        echo "key_buffer_size = 16K"
        echo "table_open_cache = 4"
        echo "sort_buffer_size = 64K"
        echo "read_buffer_size = 256K"
        echo "read_rnd_buffer_size = 256K"
        echo "net_buffer_length = 2K"
        echo "thread_stack = ${MARIADB55_THREAD_STACK_SIZE}"
    # 64 - 128 MB RAM
    elif [ "${MARIADB55_MEMORY_OPT}" = "medium" ]
    then
        echo "key_buffer_size = 16M"
        echo "table_open_cache = 64"
        echo "sort_buffer_size = 512K"
        echo "net_buffer_length = 8K"
        echo "read_buffer_size = 256K"
        echo "read_rnd_buffer_size = 512K"
        echo "myisam_sort_buffer_size = 8M"
        echo "query_cache_size = 8M"
        echo "thread_stack = ${MARIADB55_THREAD_STACK_SIZE}" # remove warning message
    # > 512 MB RAM
    elif [ "${MARIADB55_MEMORY_OPT}" = "large" ]
    then
        echo "key_buffer_size = 256M"
        echo "table_open_cache = 256"
        echo "sort_buffer_size = 1M"
        echo "read_buffer_size = 1M"
        echo "read_rnd_buffer_size = 4M"
        echo "myisam_sort_buffer_size = 64M"
        echo "thread_cache_size = 8"
        echo "query_cache_size = 16M"
        echo "thread_stack = ${MARIADB55_THREAD_STACK_SIZE}" # remove warning message
    # > 1024 MB RAM
    elif [ "${MARIADB55_MEMORY_OPT}" = "huge" ]
    then
        echo "key_buffer_size = 384M"
        echo "table_open_cache = 512"
        echo "sort_buffer_size = 2M"
        echo "read_buffer_size = 2M"
        echo "read_rnd_buffer_size = 8M"
        echo "myisam_sort_buffer_size = 64M"
        echo "thread_cache_size = 8"
        echo "query_cache_size = 32M"
        echo "thread_stack = ${MARIADB55_THREAD_STACK_SIZE}" # remove warning message
    fi
    echo "max_allowed_packet = ${MARIADB55_MAX_ALLOWED_PACKET}M"
    echo "max_connections = ${MARIADB55_MAX_CONNECTIONS}"
    # binary logging is required for replication:
    if [ "${MARIADB55_ACTIVATE_BINLOG}" = "yes" ]
    then
        echo "log-bin = mysql-bin"
        echo "max_binlog_size = ${MARIADB55_MAX_BINLOG_SIZE}"
    fi
    echo "log-error = /var/log/mysqld/55/mysqld.log"
    echo "server-id = 1"
    # character set:
    echo "character-set-server = $(echo "${MARIADB55_DEFAULT_COLLATION}" | cut -f 1 -d '_')"
    echo "collation-server = ${MARIADB55_DEFAULT_COLLATION}"

    echo '#ssl       = 1'
    echo '#ssl-ca    = /etc/mysql/ca-cert.pem'
    echo '#ssl-cert  = /etc/mysql/server-cert.pem'
    echo '#ssl-key   = /etc/mysql/server-key.pem'
    echo
    echo "[mysqldump]"
    echo "quick"
    echo "max_allowed_packet = 32M"
    echo
    echo "[mysql]"
    echo "no-auto-rehash"
    # Remove the next comment character if you are not familiar with SQL
    #safe-updates
    echo
    if [ "${MARIADB55_MEMORY_OPT}" = "small" ]
    then
        echo "[isamchk]"
        echo "key_buffer_size = 8M"
        echo "sort_buffer_size = 8M"
        echo
        echo "[myisamchk]"
        echo "key_buffer_size = 8M"
        echo "sort_buffer_size = 8M"
    elif [ "${MARIADB55_MEMORY_OPT}" = "medium" ]
    then
        echo "[isamchk]"
        echo "key_buffer_size = 20M"
        echo "sort_buffer_size = 20M"
        echo "read_buffer = 2M"
        echo "write_buffer = 2M"
        echo
        echo "[myisamchk]"
        echo "key_buffer_size = 20M"
        echo "sort_buffer_size = 20M"
        echo "read_buffer = 2M"
        echo "write_buffer = 2M"
    elif [ "${MARIADB55_MEMORY_OPT}" = "large" ]
    then
        echo "[isamchk]"
        echo "key_buffer_size = 128M"
        echo "sort_buffer_size = 128M"
        echo "read_buffer = 2M"
        echo "write_buffer = 2M"
        echo
        echo "[myisamchk]"
        echo "key_buffer_size = 128M"
        echo "sort_buffer_size = 128M"
        echo "read_buffer = 2M"
        echo "write_buffer = 2M"
    elif [ "${MARIADB55_MEMORY_OPT}" = "huge" ]
    then
        echo "[isamchk]"
        echo "key_buffer_size = 256M"
        echo "sort_buffer_size = 256M"
        echo "read_buffer = 2M"
        echo "write_buffer = 2M"
        echo
        echo "[myisamchk]"
        echo "key_buffer_size = 256M"
        echo "sort_buffer_size = 256M"
        echo "read_buffer = 2M"
        echo "write_buffer = 2M"
    fi
    echo
    echo "[mysqlhotcopy]"
    echo "interactive-timeout"
    echo
    echo "!includedir /etc/my55.cnf.d"
    } > /etc/mysql/my55.cnf

    # Set rights

    chown root:root /etc/mysql/my55.cnf
    chmod 0644 /etc/mysql/my55.cnf
}
# ---------------------------------------------------------------------------
# check for backup folder exists or create
# ---------------------------------------------------------------------------
check_backup_folder()
{
    _backup_target='/srv/mysql_backup/55'

    if [ "${MARIADB55_BACKUP_TARGET}" != "${_backup_target}" ]
    then
        _backup_target="${MARIADB55_BACKUP_TARGET}"

        if [ -n "$MARIADB55_BACKUP_MOUNT" ]
        then
            if [ ! -d "$MARIADB55_BACKUP_TARGET" ]
            then
                eval ${MARIADB55_BACKUP_MOUNT}
                _backup_mount=true
            fi
        fi
    fi

    mkdir -p ${_backup_target}
#    chmod 0770 ${_backup_target}
#    chown -R mysql:${MARIADB55_BACKUP_GROUP:-mysql} ${_backup_target}

    if "${_backup_mount:-false}"
    then
        eval ${MARIADB55_BACKUP_UMOUNT}
    fi

    # add mysql to backup group
    if [ -n "${MARIADB55_BACKUP_GROUP}" ]
    then
        /var/install/bin/add-user-to-additional-group --user mysql \
            --group ${MARIADB55_BACKUP_GROUP}
        chmod 0750 ${_backup_target}
    else
        # get old config
        old_config_file=$(find /tmp -type f -name 'edit-mariadb55-*')

        if [ -n "${old_config_file}" ]
        then
            old_group=$(grep "^MARIADB55_BACKUP_GROUP" ${old_config_file} |
                        sed "s/^MARIADB55_BACKUP_GROUP=['\"]//; s/['\"].*$//")
            if [ -n "${old_group}" ]
            then
                /var/install/bin/remove-user-from-additional-group --user mysql \
                    --group ${old_group}
            fi
        fi
        chmod 0770 ${_backup_target}
    fi

    chown -R mysql:${MARIADB55_BACKUP_GROUP:-mysql} ${_backup_target}

}

# ---------------------------------------------------------------------------
# update path to php5 mysql.sock
# ---------------------------------------------------------------------------
check_php()
{
    apache2_php5=$(eisman check apache2_php5)

    if [ "${apache2_php5}" = "installed" ] &&
       [ ${MARIADB55_CONNECT_PORT:-0} = 3306 ]
    then
        __socket=$(grep 'PHP5_EXT_MYSQL_SOCKET' /etc/config.d/apache2_php5 |
                sed "s|#.*$||; s/^.*=['\"]\(.*\)['\"].*$/\1/")

        __socket_new="/run/mysql/55/mysql.sock"

        if [ -n "${__socket}" ] &&
           [ "${__socket}" != "${__socket_new}" ]
        then
            sed -i "s|${__socket}|${__socket_new}|" /etc/config.d/apache2_php5
            apache2_start=true
        fi
    fi

    if ${apache2_start:-false}
    then
        apache2_start=$(grep '^START_APACHE2' /etc/config.d/apache2 |
                        sed "s|#.*$||; s/^.*=['\"]\(.*\)['\"].*$/\1/")

        if [ "${apache2_start}" = "yes" ]
        then
            /var/install/config.d/apache2_php5.sh >/dev/null 2>&1
        fi
    fi
}

# ---------------------------------------------------------------------------
# check for default /etc/my.cnf and create link to /etc/my.cnf
# ---------------------------------------------------------------------------
check_my_conf()
{
    if [ ${MARIADB55_CONNECT_PORT:-0} = 3306 ]
    then
        cd /etc
        rm -f my.cnf
        ln -sf mysql/my55.cnf my.cnf
    else
        my_server=$(find /etc/config.d  -maxdepth 1 \
                    -name 'mariadb' -printf '%f\n' -o \
                    -name 'mariadb[0-9]*' -printf '%f\n' -o \
                    -name 'mysql' -printf '%f\n' -o \
                    -name 'mysql[0-9]*' -printf '%f\n')

        for SERVER in ${my_server}
        do
            . /etc/config.d/${SERVER}
            var_prefix=$(echo ${SERVER} | tr [:lower:] [:upper:])

            eval connect_port='${'${var_prefix}'_CONNECT_PORT}'

            if [ ${connect_port:-0} = 3306 ]
            then
                port_used=true
                break
            fi
        done

        if ! ${port_used:-false}
        then
            rm -f /etc/my.cnf
        fi
    fi
}

# ---------------------------------------------------------------------------
# check for default info
# ---------------------------------------------------------------------------
check_default_info()
{
    if [ ${MARIADB55_CONNECT_PORT:-0} = 3306 ]
    then
        mkdir -p /var/lib/mysql
        rm -f /var/lib/mysql/defaults.info
        cp /srv/mysql/defaults.info /var/lib/mysql/defaults.info
        sed -i 's|${my_path}|55|g' /var/lib/mysql/defaults.info
    fi
}

# ---------------------------------------------------------------------------
# main
# ---------------------------------------------------------------------------
# change working path
cd /tmp

# Read config file
. /etc/config.d/mariadb55

# create or remove cron job
if [ "${START_MARIADB55}" = "yes" ] &&
   [ ${MARIADB55_BACKUP_N:-0} -gt 0 ]
then
    idx=1
    while [ ${idx} -le ${MARIADB55_BACKUP_N:-0} ]
    do
        eval active='${MARIADB55_BACKUP_'${idx}'_ACTIVE}'

        if [ "${active}" = "yes" ]
        then
            create_schedule
            backup_schedule=true
#            add_cron_job
            break
        fi

    idx=$((${idx} + 1))
    done
    check_backup_folder
else
        remove_schedule
fi

# check for new installation
if [ -f "/srv/mysql/55/new-install" ]
then
    _new_install=true
    rm  -f "/srv/mysql/55/new-install"
fi

# set datadir
if [ -n "${MARIADB55_DATADIR}" ]
then
    if "${_new_install:-false}"
    then
        _mysql_datadir="${MARIADB55_DATADIR}"
    else
        if [ -d "${_default_datadir}" ]
        then
            _mysql_datadir="${_default_datadir}"
        elif [ -d  "${MARIADB55_DATADIR}" ]
        then
            _mysql_datadir="${MARIADB55_DATADIR}"
        fi
    fi
else
    _mysql_datadir="${_default_datadir}"
fi

# make my.cnf file
make_mysql_conf

check_php
check_my_conf
check_default_info

# check for update without database upgrade
if [ -f "/srv/mysql/55/update-install" ]
then
    /var/install/bin/mysql-common-tools-update-db --project mariadb55
fi

# if new install, install in MYSQL_DATADIR
if "${_new_install:-false}"
then
    mkdir -p -m 775 "${_mysql_datadir}"
    # start MySQL internal installscript
    /usr/lib/mysql/55/bin/mysql_install_db \
       --basedir=/usr/lib/mysql/55 \
       --defaults-file=/etc/mysql/my55.cnf >/dev/null 2>&1

    chown -R mysql:mysql "${_mysql_datadir}"

    my_version="$(/usr/lib/mysql/55/bin/mysqld --version | /usr/bin/gawk '/Ver/ {print $3}' | /usr/bin/cut -d'.' -f-2)"

    echo "MYSQL_OPTION='--skip-grant-tables --skip-networking'" >/etc/sysconfig/${package_name}
    /usr/sbin/service --msg=" * Starting MariaDB/MySQL database: server ${my_version} insecure" start ${package_name}.service
    rm -f /etc/sysconfig/${package_name}

    # create root passwd
    echo
    _ask_tmpfile=$(${MKTEMP} -t .XXXXXXXXXXXXX)
    ${ASK} "Set Password for MariaDB/MySQL user root:" "yes"  >${_ask_tmpfile}
    rc=${?}
    read answer < ${_ask_tmpfile}
    ${RM} -f ${_ask_tmpfile}

    if [ "${answer}" = "yes" ]
    then
        # set root password
        /var/install/bin/mysql-common-tools-userpass -nu=root mariadb55
    fi
fi

# ---------------------------------------------------------------------------
# create dir and fix owner and rights
# ---------------------------------------------------------------------------
mkdir -p /srv/mysql
chmod 0775 /srv/mysql
chown mysql:mysql /srv/mysql

mkdir -p ${_mysql_datadir}/{mysql,test}
chmod 0770 ${_mysql_datadir}/{mysql,test}
chown -R mysql:mysql ${_mysql_datadir}/{mysql,test}

# ---------------------------------------------------------------------------
# create/remove startup link
# ---------------------------------------------------------------------------
if [ "${START_MARIADB55}" = "yes" ]
then
    start_mariadb
else
    stop_mariadb
    remove_schedule
fi

exit 0
# ---------------------------------------------------------------------------
# end
# ---------------------------------------------------------------------------
